#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "smhiggs.h"

/* 
   This program implements an SMH calculation of the Higgs self
   coupling lambda given a specified Higgs pole mass Mh. The computation 
   uses equations (2.43), (2.46)-(2.48), and (3.2)-(3.4) of the SMH preprint. 
   It takes 7 or 8 command line input values:

   Mh v yt g3 g gp Q [loops]

   (in that order) and prints the value of lambda. The argument loops is 
   optional, with allowed values:

   0    tree level
   1    1-loop level
   1.5  1-loop plus 2-loop QCD
   2    2-loop level
   2.5  2-loop plus leading 3-loop QCD
   3    2-loop plus leading 3-loop

   If loops is omitted, the default value is 3.

   Dimensionful quantities are in appropriate powers of GeV. General
   and timing information for the computation are also printed.

   Compile as, e.g.,

   gcc -DTSIL_SIZE_LONG -o calc_lambda calc_lambda.c -L. -lsmh -ltsil -lm

   (assuming libsmh.a and libtsil.a are in the current directory, and
   these were compiled using long double floating point data sizes)
   and run as, e.g.,

   ./calc_lambda 125.6 247.0 0.936 1.167 0.648 0.358 173.1

   for the result at full 2-loop and leading 3-loop order, or

   ./calc_lambda 125.6 247.0 0.936 1.167 0.648 0.358 173.1 2.5

   for the result at full 2-loop and leading QCD 3-loop order.
*/

long double strtold (const char *, char **);

int main (int argc, char *argv[])
{
  float loops = 3.;
  TSIL_REAL Mh_in, result;
  clock_t t0, t1;

  if (argc != 8 && argc != 9)
    SMH_Error ("main", "Incorrect number of arguments; should be 7 or 8:\nMh, v, yt, g3, g, gp, Q, with an optional loop order.\n", 1);

  /* Get input parameters */
  Mh_in = (TSIL_REAL) strtold(argv[1], (char **) NULL);
  SMH_v = (TSIL_REAL) strtold(argv[2], (char **) NULL);
  SMH_yt = (TSIL_REAL) strtold(argv[3], (char **) NULL);
  SMH_g3 = (TSIL_REAL) strtold(argv[4], (char **) NULL);
  SMH_g = (TSIL_REAL) strtold(argv[5], (char **) NULL);
  SMH_gp = (TSIL_REAL) strtold(argv[6], (char **) NULL);
  SMH_Q = (TSIL_REAL) strtold(argv[7], (char **) NULL);

  if (argc == 9)
    loops = (float) atof (argv[8]);

  /* Check input parameters for sanity: */
  SMH_Check_Mh_range (Mh_in);
  SMH_Check_vev_range ();
  SMH_Check_yt_range ();
  SMH_Check_g3_range ();
  SMH_Check_g_range ();
  SMH_Check_gp_range ();
  SMH_Check_Q_range ();

  /* Evaluate functions */
  t0 = clock ();

  result = SMH_Find_lambda (Mh_in, loops);

  t1 = clock ();

  SMH_Version ();
  printf("\nlambda(loops = %3.1f) = %Lf\n", loops, result);
  printf("\nTotal calculation time (s): %lf\n",
	 difftime(t1, t0)/CLOCKS_PER_SEC);

  return 0;
}
