#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "smhiggs.h"

/* 
   This program implements renormalization group running of all
   standard model parameters from one specified scale to another. It
   takes 9 or 10 command line inputs

   lambda yt g3 g gp vev m2 Qinitial Qfinal [loops]

   (in that order) and prints the values of 
   lambda, yt, g3, g, gp, v, and m2 at the scale Qfinal. 
   The last input loops is the loop order of the RG running, and is 
   optional. Allowed values for loops are 1,2,3. By default, loops = 3,
   and full 3-loop beta functions are used.  General and timing 
   information for the computation are also printed.

   Compile as, e.g.

   gcc -DTSIL_SIZE_LONG -o rgrun calc_rgrun.c -L. -lsmh -ltsil -lm

   (assuming libsmh.a and libtsil.a are in the current directory, and
   these were compiled using long double floating point data sizes)
   and run as, e.g.,

   ./rgrun 0.127 0.936 1.167 0.648 0.358 247. -8716.09 173.1 250.0

   for full 3-loop running, or 

   ./rgrun 0.127 0.936 1.167 0.648 0.358 247. -8716.09 173.1 250.0 2

   for 2-loop running.
*/

long double strtold (const char *, char **);

int main (int argc, char *argv[])
{
  TSIL_REAL Qf;
  int loops = 3;
  clock_t t0, t1;

  /* Suppress warning messages */
/*   fclose (stderr); */

  if (argc != 10 && argc != 11)
    SMH_Error ("main", "Incorrect number of arguments; should be 9 or 10:lambda, yt, g3, g, gp, v, m2, Qi, Qf, and an optional loop order.", 1);

  /* Get input parameters */
  SMH_lambda = (TSIL_REAL) strtold(argv[1], (char **) NULL);
  SMH_yt = (TSIL_REAL) strtold(argv[2], (char **) NULL);
  SMH_g3 = (TSIL_REAL) strtold(argv[3], (char **) NULL);
  SMH_g = (TSIL_REAL) strtold(argv[4], (char **) NULL);
  SMH_gp = (TSIL_REAL) strtold(argv[5], (char **) NULL);
  SMH_v = (TSIL_REAL) strtold(argv[6], (char **) NULL);
  SMH_m2 = (TSIL_REAL) strtold(argv[7], (char **) NULL);
  SMH_Q = (TSIL_REAL) strtold(argv[8], (char **) NULL);
  Qf = (TSIL_REAL) strtold(argv[9], (char **) NULL);

  /* Check input parameters for sanity: */
/*   SMH_Check_lambda_range (); */
/*   SMH_Check_yt_range (); */
/*   SMH_Check_g3_range (); */
/*   SMH_Check_g_range (); */
/*   SMH_Check_gp_range (); */
/*   SMH_Check_vev_range (); */
/*   SMH_Check_m2_range (); */
/*   SMH_Check_Q_range (); */

  SMH_Update ();

  if (argc == 11)
    loops = (int) atoi(argv[10]);

  SMH_Version ();

  printf("\nInitial model parameters:\n");
  SMH_DisplayParameters ();

  t0 = clock ();

  SMH_RGrun (Qf, loops);

  t1 = clock ();

  printf("\nFinal model parameters:\n");
  SMH_DisplayParameters ();

  printf("\nTotal calculation time (s): %lf\n",
	 difftime(t1, t0)/CLOCKS_PER_SEC);

  return 0;
}
