#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "smhiggs.h"

/* 
   This program implements an SMH calculation of the Higgs vev,
   obtained by minimizing the Landau gauge effective potential, after 
   resummation of Goldston boson contributions, at full 2-loop order with 
   leading 3-loop contributions (by default). The computation uses equations 
   (4.18)-(4.21) of arXiv:1406.2355. It takes 7 or 8 command-line inputs:

   m2 lambda yt g3 g gp Q [loops]

   (in that order) and prints the value of the vev. The argument loops is
   the loop order of the calculation, and is optional. Permissible values
   are 0,1,2,3. If loops is omitted, the default value is 3.

   Dimensionful quantities are in appropriate powers of GeV. General and timing 
   information for the computation are also printed.

   Compile as, e.g.

   gcc -DTSIL_SIZE_LONG -o calc_vev calc_vev.c -L. -lsmh -ltsil -lm

   (assuming libsmh.a and libtsil.a are in the current directory, and
   these were compiled using long double floating point data sizes)
   and run as, e.g.,

   ./calc_vev -8716.09 0.127 0.936 1.167 0.648 0.358 173.1

   for the result with full 2-loop and leading 3-loop contributions, or 

   ./calc_vev -8716.09 0.127 0.936 1.167 0.648 0.358 173.1 2

   for the result with only full 2-loop contributions.
*/

long double strtold (const char *, char **);

int main (int argc, char *argv[])
{
  TSIL_REAL result;
  int loops = 3;
  clock_t t0, t1;

  if (argc != 8 && argc != 9)
    SMH_Error ("main", "Incorrect number of arguments; should be 7 or 8:\nm2, lambda, yt, g3, g, gp, Q, and an optional loop order.\n", 1);

  /* Get input parameters */
  SMH_m2 = (TSIL_REAL) strtold(argv[1], (char **) NULL);
  SMH_lambda = (TSIL_REAL) strtold(argv[2], (char **) NULL);
  SMH_yt = (TSIL_REAL) strtold(argv[3], (char **) NULL);
  SMH_g3 = (TSIL_REAL) strtold(argv[4], (char **) NULL);
  SMH_g = (TSIL_REAL) strtold(argv[5], (char **) NULL);
  SMH_gp = (TSIL_REAL) strtold(argv[6], (char **) NULL);
  SMH_Q = (TSIL_REAL) strtold(argv[7], (char **) NULL);

  if (argc == 9)
    loops = (int) atoi(argv[8]);

  /* Check input parameters for sanity: */
  SMH_Check_m2_range ();
  SMH_Check_lambda_range ();
  SMH_Check_yt_range ();
  SMH_Check_g3_range ();
  SMH_Check_g_range ();
  SMH_Check_gp_range ();
  SMH_Check_Q_range ();

  t0 = clock ();

  result = SMH_Find_vev (loops);

  t1 = clock ();

  SMH_Version ();
  printf("\nvev = %Lf\n", result);
  printf("\nTotal calculation time (s): %lf\n",
	 difftime(t1, t0)/CLOCKS_PER_SEC);

  return 0;
}
