/* Evaluation and minimization of the SM effective potential in Landau
   gauge at full two-loop order including partial 3-loop results. */

#include "smh_internal.h"

/* Parameters governing iteration in SMH_Find_vev */
#define MAXITERS 50
#define TOL      1.e-12

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
/* Evaluates the effective potential for current parameter values and */
/* at the specified loop order. */

TSIL_REAL SMH_Veff (int loopOrder)
{
  TSIL_COMPLEX V1, V2, V3, result;
  TSIL_COMPLEX V2S, V2FFS, V2SV, V2FFV, V2gauge;
  TSIL_COMPLEX AG, AH, At;
  TSIL_REAL T2, lnbarT, lnbarT2, lnbarT3;

  if (loopOrder < 0 || loopOrder > 3) exit(0);

  /* Tree level: */
  result = (m2/2.0) * v2 + (k/4.0) * v4;

  if (loopOrder >= 1) {
    V1 = (1.0/4.0) * H * H * (LnBar(H) - 3.0/2.0) + 
      3.0/4.0 * G * G * (LnBar(G) - 3.0/2.0) -
      3.0 * T * T * (LnBar(T) - 3.0/2.0) + 
      3.0/2.0 * W * W * (LnBar(W) - 5.0/6.0) +
      3.0/4.0 * Z * Z * (LnBar(Z) - 5.0/6.0);

    result += ONELOOPFACTOR * V1;
  }
  if (loopOrder >= 2) {

    AG = TSIL_A (G, Q2);
    AH = TSIL_A (H, Q2);
    At = TSIL_A (T, Q2);

    V2S = (3.0*k*(5.0*AG*AG + 2.0*AG*AH + AH*AH))/4.0 - 
      3.0*k2*v2*(TSIL_I2(H, G, G, Q2) + TSIL_I2(H, H, H, Q2));

    V2FFS = 3.0*yt2*(-2.0*AG*At - AH*At + At*At + 
		     (-G + T) * TSIL_I2 (T, G, 0.0, Q2) -
		     (G * TSIL_I2 (T, T, G, Q2))/2.0 + 
		     (-H/2.0 + 2.0*T) * TSIL_I2 (T, T, H, Q2));

    V2SV = 
      (g2*gp2*SMH_fSSV (G, G, 0))/(2*(g2 + gp2)) + 
      (g2*SMH_fSSV (G, G, W))/4 +
      ((g2 - gp2)*(g2 - gp2)*SMH_fSSV (G, G, Z))/(8*(g2 + gp2)) +
      (g2*SMH_fSSV (H, G, W))/4 +
      ((g2 + gp2)*SMH_fSSV (H, G, Z))/8 + (3*g2*SMH_fVS (W, G))/4 +
      (g2*SMH_fVS (W, H))/4 +
      ((3*g2)/8 - (5*gp2)/8 + gp4/(g2 + gp2))*SMH_fVS (Z, G) +
      ((g2 + gp2)*SMH_fVS (Z, H))/8 +
      (g2*gp2*W*SMH_fVVS (W, 0, G))/(g2 + gp2) + /* DGR <-- This term
						    gave trouble as
						    written due to
						    y=0 */
      (g2*W*SMH_fVVS (W, W, H))/2 +
      (g2*gp4*Z*SMH_fVVS (Z, W, G))/((g2 + gp2)*(g2 + gp2)) +
      ((g2 + gp2)*Z*SMH_fVVS (Z, Z, H))/4
      ;

    V2FFV = (4*g32 + (4*g2*gp2)/(3*(g2 + gp2)))*
      (-(T*SMH_fffV (T, T, 0)) + SMH_fFFV (T, T, 0)) +
      (g2*(8*SMH_fFFV (0, 0, W) + 3*SMH_fFFV (T, b, W) + 
	   SMH_fFFV (tau, 0, W)))/2 + 
      (4*b*gp2*(3*g2 + gp2)*SMH_fffV (b, b, Z) + 
       8*(3*g2 - gp2)*gp2*T*SMH_fffV (T, T, Z) +
       12*(g2 - gp2)*gp2*tau*SMH_fffV (tau, tau, Z) + 
       (51*g4 + 6*g2*gp2 + 83*gp4)*
       SMH_fFFV (0, 0, Z) + (9*g4 + 6*g2*gp2 + 5*gp4)*SMH_fFFV (b, b, Z) +
       (9*g4 - 6*g2*gp2 + 17*gp4)*SMH_fFFV (T, T, Z) + 
       (3*g4 - 6*g2*gp2 + 15*gp4)*
       SMH_fFFV (tau, tau, Z))/(24*(g2 + gp2));

    V2gauge = (g2*(gp2*SMH_fgauge (W, W, 0) + 
		   g2*SMH_fgauge (W, W, Z)))/(2*(g2 + gp2));

    V2 = V2S + V2FFS + V2SV + V2FFV + V2gauge;

    result += TWOLOOPFACTOR * V2;
  }
  if (loopOrder >= 3) {

    T2 = T*T;
    lnbarT = LnBar (T);
    lnbarT2 = lnbarT*lnbarT;
    lnbarT3 = lnbarT2*lnbarT;

    V3 = g34*T2*(-184*lnbarT3 + 868*lnbarT2 - 
		 1842.20084576555964953388*lnbarT + 
		 1957.33164815913965065333) +
      g3*yt2*T2*(60*lnbarT3 - 360*lnbarT2 + 
		 1220.92890957137493192195*lnbarT - 
		 780.302083273558636826600) +
      yt4*T2*(24.75*lnbarT3 + 
	      (81 + 81*TSIL_LOG (H/T) + 27*TSIL_LOG (G/T))*lnbarT2 +
	      (-971.627691424366640808027 - 54*TSIL_LOG (H/T) - 
	       54*TSIL_LOG (G/T))*lnbarT + 
	      504.510661119167364447838 + 9*TSIL_LOG (H/T) + 
	      27*TSIL_LOG (G/T));

    result += THREELOOPFACTOR * V3;
  }

  return TSIL_CREAL (result);
}

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
/* Returns:

      kappa delta1 + kappa^2 delta2 + kappa^3 delta3

   for use in Veff minimization. */

TSIL_REAL DeltaTerms (int loopOrder)
{
  TSIL_COMPLEX delta1, delta2, delta3, result;

  TSIL_REAL 
    cIttZ,cIWWZ,cIhZZ,cIhWW,cIhtt,cIhhh,cI00Z,cI00W,cI00t,cI00h,
    cI0hZ,cI0hW,cI0tW,cI0WZ,cAtAt,cAtAh,cAtAW,cAtAZ,cAhAh,cAhAW,
    cAhAZ,cAWAW,cAWAZ,cAZAZ,cAh,cAt,cAW,cAZ,c1;

  TSIL_COMPLEX 
    IttZ,IWWZ,IhZZ,IhWW,Ihtt,Ihhh,I00Z,I00W,I00t,I00h,I0hZ,I0hW,
    I0tW,I0WZ,AZ,AW,Ah,At;

  TSIL_COMPLEX lnbarh,lnbart,lnbart2,lnbart3;

  if (0 == loopOrder) return 0;

  /* One loop terms */
  Ah = TSIL_A (h, Q2);
  At = TSIL_A (T, Q2);
  AW = TSIL_A (W, Q2);
  AZ = TSIL_A (Z, Q2);

  delta1 = 3*k*Ah - 6*yt2*At + (3*g2*AW)/2 + (3*(g2 + gp2)*AZ)/4 +
    ((3*g4 + 2*g2*gp2 + gp4)*v2)/8;  

  result = ONELOOPFACTOR * delta1;

  if (loopOrder > 1) {  /* Add the two loop terms */

    cIttZ = -(9*g6 + 3*g4*gp2 + 11*g2*gp4 + 17*gp6 - 18*g4*yt2 - 
	      132*g2*gp2*yt2 + 14*gp4*yt2)/(12*(g2 + gp2));

    cIWWZ = ((3*g2 - gp2)*(33*g4 + 22*g2*gp2 + gp4))/(8*(g2 + gp2));

    cIhZZ = (3*g6 + 9*g4*gp2 + 9*g2*gp4 + 3*gp6)/(32*(g2 + gp2 - 2*k)) +
      (-15*g4 - 30*g2*gp2 - 15*gp4 + 44*g2*k + 44*gp2*k - 112*k2)/16;

    cIhWW = (-27*g6 + 148*g4*k - 400*g2*k2 + 448*k3)/(16*g2 - 32*k);

    cIhtt = (-36*k*yt2 + 27*yt4)/2;
      
    cIhhh = -15*k2;

    cI00Z = (9*g4*k)/(2*g2 + 2*gp2) + (-63*g4 - 6*g2*gp2 - 103*gp4 - 
				       36*g2*k)/12;

    cI00W = (-18*g4 + 15*g2*k)/2 + (-9*g6*k - 6*g4*gp2*k)/
      (2*g4 + 4*g2*gp2 + 2*gp4);
       
    cI00t = -12*k*yt2;

    cI00h = -6*k2;

    cI0hZ = (3*g2*k + 3*gp2*k + 24*k2)/4;
     
    cI0hW = (3*g2*k + 24*k2)/2;

    cI0tW = -3*g4 + 3*g2*yt2 + 6*yt4;
     
    cI0WZ = (12*g2*k + 3*gp2*k)/2 + (12*g6*k + 9*g4*gp2*k)/
      (2*g4 + 4*g2*gp2 + 2*gp4);

    cAtAt = ((73*g2)/3 + 96*g32 + (17*gp2)/3 - (64*g4)/(3*g2 + 3*gp2) + 6*k +
	     15*yt2)/v2;
      
    cAtAh = (-9*yt2)/v2;
       
    cAtAW = (-12*(g2 - yt2))/v2;

    cAtAZ = (-2*(9*g4 - 6*g2*gp2 + 17*gp4))/(3*(g2 + gp2)*v2);

    cAhAh = 0;

    cAhAW = (9*g4 - 44*g2*k + 64*k2)/((4*g2 - 8*k)*v2);

    cAhAZ = ((3*g2)/4 + (3*gp2)/4 + (3*g4 + 6*g2*gp2 + 3*gp4)/
	     (8*(g2 + gp2 - 2*k)) - 4*k)/v2;

    cAWAW = ((24*g4)/(g2 + gp2) - (3*g4)/(4*g2 - 8*k) + (-35*g2 + gp2 + 
							 14*k)/2)/v2;

    cAWAZ = (15*g6 + 5*g4*gp2 - 11*g2*gp4 - gp6 + 48*g4*k + 48*g2*gp2*k +
	     6*gp4*k)/((g2 + gp2)*(g2 + gp2)*v2);

    cAZAZ = ((-3*g4 - 6*g2*gp2 - 3*gp4)/(8*(g2 + gp2 - 2*k)) +
	     (-g2 - gp2 + 7*k)/2)/v2;

    cAh = (-3*g6)/(16*(g2 - 2*k)) + (-3*g6 - 9*g4*gp2 - 9*g2*gp4 - 3*gp6)/
      (32*(g2 + gp2 - 2*k)) + (18*g4 + 12*g2*gp2 + 6*gp4 - 21*g2*k -
			       7*gp2*k + 96*k2 + 36*k*yt2 - 42*yt4)/4;

    cAt = (64*g4*yt2)/(3*g2 + 3*gp2) + (-91*g2*yt2 + 48*g32*yt2 + 11*gp2*yt2 +
					72*k*yt2 - 36*yt4)/3;

    cAW = (-3*g6)/(8*(g2 - 2*k)) + (-12*g6 - 3*g4*k)/(g2 + gp2) +
      (605*g4 + 39*g2*gp2 - 192*g2*k - 252*g2*yt2)/24;

    cAZ = (-3*g6 - 9*g4*gp2 - 9*g2*gp4 - 3*gp6)/(16*(g2 + gp2 - 2*k)) +
      (18*g6 - 32*g4*yt2)/(3*g2 + 3*gp2) +
      (29*g4 + 192*g2*gp2 + 455*gp4 - 552*g2*k + 24*gp2*k + 260*g2*yt2 -
       380*gp2*yt2)/48;

    c1 = v2*((199*g6)/64 - (773*g2*gp4)/192 - (497*gp6)/192 +
	     (9*g8)/(32*(g2 - 2*k)) + (9*g8 + 36*g6*gp2 + 54*g4*gp4 + 
		   36*g2*gp6 + 9*gp8)/(64*(g2 + gp2 - 2*k)) + 
	     (7*g2*gp2*k)/8 + 6*g2*k2 + 2*gp2*k2 -
	     60*k3 + (23*g2*gp2*yt2)/8 - 18*k2*yt2 +
	     (-551*g4*gp2 + 1116*g4*k - 324*g4*yt2)/192 + 
	     (3*gp4*k - 91*gp4*yt2)/
	     48 + (41*g2*yt4)/3 + 24*g32*yt4 + (5*gp2*yt4)/6 + 6*k*yt4 +
	     (72*g8 + 9*g6*k - 256*g4*yt4)/(24*g2 + 24*gp2) + 9*yt6);

    IttZ = TSIL_I2 (T, T, Z, Q2);
    IWWZ = TSIL_I2 (W, W, Z, Q2);
    IhZZ = TSIL_I2 (h, Z, Z, Q2);
    IhWW = TSIL_I2 (h, W, W, Q2);
    Ihtt = TSIL_I2 (h, T, T, Q2);
    Ihhh = TSIL_I2 (h, h, h, Q2);
    I00Z = TSIL_I2 (0, 0, Z, Q2);
    I00W = TSIL_I2 (0, 0, W, Q2);
    I00t = TSIL_I2 (0, 0, T, Q2);
    I00h = TSIL_I2 (0, 0, h, Q2);
    I0hZ = TSIL_I2 (0, h, Z, Q2);
    I0hW = TSIL_I2 (0, h, W, Q2);
    I0tW = TSIL_I2 (0, T, W, Q2);
    I0WZ = TSIL_I2 (0, W, Z, Q2);

    delta2 = cIttZ * IttZ +
      cIWWZ * IWWZ +
      cIhZZ * IhZZ +
      cIhWW * IhWW +
      cIhtt * Ihtt +
      cIhhh * Ihhh +
      cI00Z * I00Z +
      cI00W * I00W +
      cI00t * I00t +
      cI00h * I00h +
      cI0hZ * I0hZ +
      cI0hW * I0hW +
      cI0tW * I0tW +
      cI0WZ * I0WZ +
      cAtAt * At*At +
      cAtAh * At*Ah +
      cAtAW * At*AW +
      cAtAZ * At*AZ +
      cAhAh * Ah*Ah +
      cAhAW * Ah*AW +
      cAhAZ * Ah*AZ +
      cAWAW * AW*AW +
      cAWAZ * AW*AZ +
      cAZAZ * AZ*AZ +
      cAh * Ah +
      cAt * At +
      cAW * AW +
      cAZ * AZ +
      c1;

    result += TWOLOOPFACTOR * delta2;
  }
  if (loopOrder > 2) {  /* Add in the three loop terms... */

    lnbarh = Ah/(2*k*v2) + 1;
    lnbart = 2*At/(yt2*v2) + 1;
    lnbart2 = lnbart*lnbart;
    lnbart3 = lnbart*lnbart2;

    delta3 = v2*(g34*yt4*(1036.2312252763598258864 
			  - 974.200845765559649533875*lnbart 
			  + 592.*lnbart2 - 184.*lnbart3) 
		 + g32*yt6*(-169.83762848787117086562 
			    + 860.928909571374931921951*lnbart 
			    - 270.*lnbart2 + 60.*lnbart3)
		 + yt8*(-82.91199779628403181268 
			- 753.018878221098564951523*lnbart 
			+ 36.*lnbarh*lnbart + 82.125*lnbart2 
			+ 54.*lnbarh*lnbart2 - 56.25*lnbart3));

    result += THREELOOPFACTOR * delta3;
  }

  return result;
}

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
/* Determines Higgs lagrangian mass^2 parameter, by minimizing the
   effective potential at the specified loop order. */

TSIL_REAL SMH_Find_m2 (int loopOrder)
{
  char funcname[] = "Find_m2";

  if (loopOrder < 0 || loopOrder > 3)
    SMH_Error (funcname, "Invalid loop order specified.", 2);

  SMH_Update ();

  /* Iteration is not required; we just calculate m2: */
  return -k*v2 - DeltaTerms (loopOrder);
}

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
/* Determines the Higgs field vev, by minimizing the effective */
/* potential at the specified loop order.  */

TSIL_REAL SMH_Find_vev (int loopOrder)
{
  TSIL_REAL oldv, lastv, newv;
  int i;
  char funcname[] = "Find_vev";

  if (loopOrder < 0 || loopOrder > 3)
    SMH_Error (funcname, "Invalid loop order specified.", 2);

  /* Save original value. */
  oldv = v;

  /* Make an initial guess, in case v wasn't set yet. */
  v = 247.;

  /* Then iterate to find v2... */
  for (i=0; i<MAXITERS; i++) {

    /* Update parameter values */
    SMH_Update ();

    lastv = v;
    v = newv = TSIL_SQRT(-(m2 + DeltaTerms (loopOrder))/k);

    /* Check for convergence */
    if (TSIL_FABS((v - lastv)/lastv) < TOL) {
      v = oldv;
      SMH_Update ();
      return newv;
    }
  }

  /* If we get here, MAXITERS was exceeded. Issue a warning and save
     the last value obtained. */
  SMH_Warn (funcname, "Too many iterations! Convergence not assured.");
  v = oldv;
  SMH_Update ();

  return newv;
}

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
