/*
   Produces scalar field VEV vs. MSbar renormalization scale data,
   determined by minimization of the scalar effective potential.

   Compile as:

   gcc -DTSIL_SIZE_LONG -o fig_vev_vs_Q fig_vev_vs_Q.c -L. -lsmh -ltsil -lm

   (assuming libsmh.a and libtsil.a are in the current directory, and
   these were compiled using long double floating point data sizes),
   and run as

   ./fig_vev_vs_Q

   The data is output in the file "FIG_vev_vs_Q.dat" in 5 columns, respectively:
   Q, v(2), v(3), v(2)/v(run), v(3)/v(run)
   where v(n) is the value of v obtained by minimizing the effective potential
   at n-loop order at the scale Q, and v(run) is obtained by directly running
   the input value of v to the scale Q using 3-loop RG equations.
*/

#include <stdio.h>
#include <math.h>
#include "smhiggs.h"

#define QSTART 90
#define QEND   270
#define QSTEP  3


int main () {

  TSIL_REAL Q, resultv2, resultv3;
  FILE *outfile;
  char outFileName[] = "FIG_vev_vs_Q.dat";

  /* Print version information: */
  SMH_Version ();

  /* Set input parameters: */
  SMH_Q  = 173.1;
  SMH_lambda = 0.12711;
  SMH_yt = 0.93558;
  SMH_g3 = 1.1666;
  SMH_g  = 0.64822;
  SMH_gp = 0.35761;
  SMH_m2 = -(93.36*93.36);
  SMH_v = 247.03931235397684;  

  /* Open output data file: */
  outfile = fopen (outFileName, "w");
  
  for (Q = QSTART; Q <= QEND; Q += QSTEP) {

    SMH_RGrun (Q, 3);

    fprintf (outfile, "%Lf", Q);

    resultv2 = SMH_Find_vev (2);
    resultv3 = SMH_Find_vev (3);

    fprintf (outfile, "\t%Lf\t%Lf", resultv2, resultv3);
    fprintf (outfile, "\t%Lf\t%Lf", resultv2/SMH_v, resultv3/SMH_v);

    fprintf (outfile, "\n");
  }

  fclose (outfile);

  return 0;
}
