/*
  General test program for SMH. 

  Compares SMH results calculated with benchmark parameters to
  expected values at two different renormalization scales.

  Compile as, e.g., 

  gcc -DTSIL_SIZE_LONG -o smh.tst smhtest.c -L. -lsmh -ltsil -lm

  (assuming libsmh.a and libtsil.a are in the current directory, and
  these were compiled using long double floating point data sizes),
  and run as
  
  ./smh.tst

  The computed and expected values will be printed, along with a
  pass/fail verdict. SMH passes the test if all computed values agree
  with the expected ones to a part in 10^8 (see parameter TOL below).
*/

#include <stdio.h>
#include <math.h>
#include "smhiggs.h"

/* 3-loop benchmark values at Q = 173.1 GeV: */
#define MHQ173   125.8176744
#define LAMQ173  0.127110
#define M2Q173  -8716.089600
#define VEVQ173  247.0393124

/* 3-loop benchmark values at Q = 250.0 GeV: */
#define MHQ250   125.7888377
#define LAMQ250  0.1200209684
#define M2Q250  -8807.365904
#define VEVQ250  246.1838560

/* Model parameters expected at Q = 250 GeV via RG running of
   benchmark model: */
#define GPRUN  0.3583542460
#define GRUN   0.6462951602
#define G3RUN  1.140814726
#define YTRUN  0.9163971286
#define LAMRUN 0.1200209684
#define M2RUN  -8813.707166
#define VEVRUN 246.0851650

/* Relative pass/fail tolerance: */
#define TOL 1e-8

enum { FAIL , PASS };

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */

int Compare (TSIL_REAL computed, TSIL_REAL expected)
{
  if (TSIL_FABS ((computed - expected)/expected) < TOL)
    return PASS;
  else
    return FAIL;
}

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */

int main () {

  TSIL_REAL Mh, lam, m2, vev;
  int result = PASS;

  SMH_Version ();

  /* Set benchmark parameters: */
  SMH_Q  = 173.10;
  SMH_gp = 0.35761;
  SMH_g  = 0.64822;
  SMH_g3 = 1.1666;
  SMH_yt = 0.93558;
  SMH_lambda = 0.12711;
  SMH_m2 = -(93.36*93.36);
  SMH_v = 247.03931235397684;

  printf ("\nAt Q = %5.1Lf\n", SMH_Q);
  Mh = SMH_Find_Mh (3);
  lam = SMH_Find_lambda (Mh, 3);
  m2 = SMH_Find_m2 (3);
  vev = SMH_Find_vev (3);

  printf("Mh     = % 13.7Lf, expected %13.7lf\n", Mh, MHQ173);
  printf("lambda = % 13.7Lf, expected %13.7lf\n", lam, LAMQ173);
  printf("m2     = %13.7Lf, expected %13.7lf\n", m2, M2Q173);
  printf("vev    = % 13.7Lf, expected %13.7lf\n", vev, VEVQ173);

  result *= Compare (Mh, MHQ173)
    * Compare (lam, LAMQ173)
    * Compare (m2, M2Q173)
    * Compare (vev, VEVQ173);

  if (result == PASS) printf("\nPASS\n");
  else printf ("\nFAIL\n");

  SMH_RGrun (250., 3);

  printf ("\nAt Q = %5.1Lf\n", SMH_Q);

  printf("Model parameters:\n");
  printf("m2     = % 13.7Lf, expected %13.7lf\n", SMH_m2, M2RUN);
  printf("lambda = % 13.7Lf, expected %13.7lf\n", SMH_lambda, LAMRUN);
  printf("yt     = % 13.7Lf, expected %13.7lf\n", SMH_yt, YTRUN);
  printf("g3     = % 13.7Lf, expected %13.7lf\n", SMH_g3, G3RUN);
  printf("g      = % 13.7Lf, expected %13.7lf\n", SMH_g, GRUN);
  printf("gp     = % 13.7Lf, expected %13.7lf\n", SMH_gp, GPRUN);
  printf("v      = % 13.7Lf, expected %13.7lf\n", SMH_v, VEVRUN);
  printf("\n");

  result *= Compare (SMH_m2, M2RUN)
    * Compare (SMH_lambda, LAMRUN)
    * Compare (SMH_yt, YTRUN)
    * Compare (SMH_g3, G3RUN)
    * Compare (SMH_g, GRUN)
    * Compare (SMH_gp, GPRUN)
    * Compare (SMH_v, VEVRUN);

  Mh = SMH_Find_Mh (3);
  lam = SMH_Find_lambda (Mh, 3);
  m2 = SMH_Find_m2 (3);
  vev = SMH_Find_vev (3);

  printf("Mh     = % 13.7Lf, expected %13.7lf\n", Mh, MHQ250);
  printf("lambda = % 13.7Lf, expected %13.7lf\n", lam, LAMQ250);
  printf("m2     = %13.7Lf, expected %13.7lf\n", m2, M2Q250);
  printf("vev    = % 13.7Lf, expected %13.7lf\n", vev, VEVQ250);

  result *= Compare (Mh, MHQ250)
    * Compare (lam, LAMQ250)
    * Compare (m2, M2Q250)
    * Compare (vev, VEVQ250);

  if (result == PASS) printf("\n=== PASS ===\n");
  else printf ("\n=== FAIL ===\n");

  return 0;
}
