/*
  Implements one- and two-loop vacuum energy functions in the MSbar
  scheme.
*/

#include "smh_internal.h"

/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */
/* hep-ph/0111209 eq. (4.16), (4.24) */

TSIL_COMPLEX SMH_fSSV (TSIL_REAL x,
		       TSIL_REAL y,
		       TSIL_REAL z)
{
  TSIL_REAL Ax, Ay, Az;

  Ax = TSIL_A (x, Q2);
  Ay = TSIL_A (y, Q2);

  if (TSIL_FABS (z/(x+y+Q2)) < TSIL_TOL) 
  {
    return (3.L * (Ax * Ay + (x + y) * TSIL_I2 (x, y, 0.L, Q2))
      - 2.L * (x * Ax + y * Ay) + (x + y)*(x + y));
  }

  Az = TSIL_A (z, Q2);

  return (( 
    (2.L * (x*y + x*z + y*z) - x*x - y*y - z*z) * TSIL_I2 (x, y, z, Q2)
    + (x-y)*(x-y) * TSIL_I2 (x, y, 0.L, Q2)
    + (y - x - z) * Ax * Az + (x - y - z) * Ay * Az 
    + z * Ax * Ay + 2.L * z * (x + y - (z/3.L)) * Az
    )/z);
}

/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */

TSIL_COMPLEX SMH_fVS (TSIL_REAL x,
		      TSIL_REAL y)
{
  TSIL_COMPLEX Ay = TSIL_A (y, Q2);

  return 3.0 * Ay * TSIL_A (x, Q2) + 2.0 * x * Ay;
}

/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */

TSIL_COMPLEX SMH_fVVS (TSIL_REAL x,
		       TSIL_REAL y,
		       TSIL_REAL z)
{
  TSIL_COMPLEX Ax, Ay, Az;

  Ax = TSIL_A (x, Q2);
  Az = TSIL_A (z, Q2);

  if (TSIL_FABS (y) > TSIL_TOL) {

    Ay = TSIL_A (y, Q2);

    return 1.0/(4*x*y) *
      ( (-x*x - y*y - z*z - 10*x*y + 2*x*z + 2*y*z) * 
	TSIL_I2 (x, y, z, Q2) +
	(x-z) * (x-z) * TSIL_I2 (0.0, x, z, Q2) + (y-z) * (y-z) * 
	TSIL_I2 (0.0, y, z, Q2) - 
	z * z * TSIL_I2 (0.0, 0.0, z, Q2) +
	(z-x-y) * Ax * Ay + y * Ax * Az + 
	x * Ay * Az ) +
      0.5 * Ax + 0.5 * Ay + 2 * Az - x - y - z;

  } else {

    return -3.0 * x/4 - z/2 - 3 * z/(4 * x) * TSIL_I2 (0.0, 0.0, z, Q2) +
      (3.0 * z/(4 * x) - 9.0/4.0) * TSIL_I2 (0.0, x, z, Q2) + 
      3.0/(4.0 * x) * Ax * Az + 2 * Az;
 }
}

/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */

TSIL_COMPLEX SMH_fFFV (TSIL_REAL x,
		       TSIL_REAL y,
		       TSIL_REAL z)
{
  TSIL_COMPLEX Ax, Ay, Az;

  if (TSIL_FABS (z) > TSIL_TOL) {

    Ax = TSIL_A (x, Q2);
    Ay = TSIL_A (y, Q2);
    Az = TSIL_A (z, Q2);

    return (1.0/z) * 
      ((x * x + y * y - 2.0 * z * z - 2.0 * x * y + x * z + y * z) * 
       TSIL_I2 (x, y, z, Q2) +
       -(x-y) * (x-y) * TSIL_I2 (0.0, x, y, Q2) + (x-y-2.0 * z) * Ax * Az 
       + (y-x-2 * z) * Ay * Az + 2.0 * z * Ax * Ay ) 
      + 2.0 * (-x - y + z/3.0) * Az - 
      2.0 * x * Ax - 2.0 * y * Ay + (x+y) * (x+y) - z * z;

  } else {
    return 0.0;
  }
}

/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */

TSIL_COMPLEX SMH_fffV (TSIL_REAL x,
		       TSIL_REAL y,
		       TSIL_REAL z)
{
  return 6.0 * TSIL_I2 (x, y, z, Q2) + 2 * (x + y + z)
    -4.0 * (TSIL_A (x, Q2) + TSIL_A (y, Q2));
}

/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */

TSIL_COMPLEX SMH_Prefgauge (TSIL_REAL x,
			    TSIL_REAL y,
			    TSIL_REAL z)
{
  TSIL_REAL x2, x3, y2, z2;
  x2 = x*x; x3 = x2*x; y2 = y*y; z2 = z*z;

  return 1.0/(4.0 * x * y * z) * 
    ((-x2 * x2 - 8 * x3 * y - 8 * x3 * z + 32 * x2 * y * z + 18 * y2 * z2) * 
     TSIL_I2 (x, y, z, Q2) +
     (y-z) * (y-z) * (y2 + 10 * y * z + z2) * TSIL_I2 (0.0, y, z, Q2) + 
     x2 * (2 * y * z - x2) * TSIL_I2 (0.0, 0.0, x, Q2) +
     (x2 - 9 * y2 - 9 * z2 + 9 * x * y + 9 * x * z + 14 * y * z) * 
     x * TSIL_A (y, Q2) * TSIL_A (z, Q2) +
     4 * x3 * y * z + 48 * x * y2 * z2 + (22 * y + 22 * z - 16 * x/3) * 
     x * y * z * TSIL_A (x, Q2));
}
/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */

TSIL_COMPLEX SMH_fgauge (TSIL_REAL x,
			 TSIL_REAL y,
			 TSIL_REAL z)
{
  TSIL_COMPLEX Ax, Ay;

  if (TSIL_FABS (z) > TSIL_TOL) {

    return SMH_Prefgauge(x, y, z) + SMH_Prefgauge(y, x, z) + 
      SMH_Prefgauge(z, y, x);

  } else {

    Ax = TSIL_A (x, Q2);
    Ay = TSIL_A (y, Q2);

    return 1.0/(4.0*x*y) * ((43.0*x*x*y + 43.0*x*y*y - 7.0*x*x*x - 
		     7.0*y*y*y) * TSIL_I2 (0.0, x, y, Q2) +
		    (2.0*y + 7.0*x) * x*x * TSIL_I2 (0.0, 0.0, x, Q2) + 
		    (2.0*x + 7.0*y) * y*y * TSIL_I2 (0.0, 0.0, y, Q2) +
		    (34.0*x*y - 7.0*x*x - 7.0*y*y) * Ax * Ay) + 
      4.0*x*x + 4.0*y*y + (35./2.)*x*y +
      - (19./3.)*(x * Ax + y * Ay) + 5.0*(y * Ax + x * Ay);
  }
}
